using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.DirectoryServices;

namespace WebService1
{
    public class LDAP
    {
        public enum LDAPResult
        {
            Authenticated = 0,
            BadDomain,
            BadUser,
            BadPassword
        }

        public enum NetworkType
        {
            ActiveDirectory,
            NovellEDirectory
        }

        private LDAP()
        {
        }

        public static LDAPResult AuthenticateLogin(string domain, string username, string password, NetworkType nettype)
        {
            AuthenticationTypes authtype = AuthenticationTypes.SecureSocketsLayer;
            if (nettype == NetworkType.ActiveDirectory)
                authtype = AuthenticationTypes.Secure;

            using (DirectoryEntry dirEntry = new DirectoryEntry("LDAP://" + domain, username, password, authtype))
            {
                try
                {
                    string s = dirEntry.Name;
                    return LDAPResult.Authenticated;
                }
                catch (Exception ex)
                {
                    if (IsFailueMessage(ex))
                        return LDAPResult.BadPassword;
                    return LDAPResult.BadDomain;
                }
            }
        }

        private static bool IsFailueMessage(Exception ex)
        {
            return ex.Message.IndexOf("Logon failure") != -1 || ex.Message.IndexOf("0x8007052E") != -1;
        }
    }
}
